"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants = require("./constants");
const forge = require("node-forge");
const path = require("path");
const minimatch = require("minimatch");
const plist = require("simple-plist");
class CloudBuildHelper {
    constructor($nsCloudErrorsService, $fs, $logger) {
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$fs = $fs;
        this.$logger = $logger;
    }
    getProvisionType(provisionData) {
        let result = "";
        if (provisionData.Entitlements['get-task-allow']) {
            result = constants.PROVISION_TYPES.DEVELOPMENT;
        }
        else {
            result = constants.PROVISION_TYPES.ADHOC;
        }
        if (!provisionData.ProvisionedDevices || !provisionData.ProvisionedDevices.length) {
            if (provisionData.ProvisionsAllDevices) {
                result = constants.PROVISION_TYPES.ENTERPRISE;
            }
            else {
                result = constants.PROVISION_TYPES.APP_STORE;
            }
        }
        return result;
    }
    getCertificateInfo(certificatePath, certificatePassword) {
        const certificateAbsolutePath = path.resolve(certificatePath);
        const certificateContents = this.$fs.readFile(certificateAbsolutePath, { encoding: 'binary' });
        const pkcs12Asn1 = forge.asn1.fromDer(certificateContents);
        const pkcs12 = forge.pkcs12.pkcs12FromAsn1(pkcs12Asn1, false, certificatePassword);
        for (let safeContens of pkcs12.safeContents) {
            for (let safeBag of safeContens.safeBags) {
                if (safeBag.attributes.localKeyId && safeBag.type === forge.pki.oids['certBag']) {
                    let issuer = safeBag.cert.issuer.getField(constants.CRYPTO.ORGANIZATION_FIELD_NAME);
                    return {
                        pemCert: forge.pki.certificateToPem(safeBag.cert),
                        organization: issuer && issuer.value,
                        validity: safeBag.cert.validity,
                        commonName: safeBag.cert.subject.getField(constants.CRYPTO.COMMON_NAME_FIELD_NAME).value,
                        friendlyName: _.head(safeBag.attributes.friendlyName)
                    };
                }
            }
        }
        this.$nsCloudErrorsService.fail(`Could not read ${certificatePath}. Please make sure there is a certificate inside.`);
    }
    isReleaseConfiguration(buildConfiguration) {
        return buildConfiguration.toLowerCase() === constants.CLOUD_BUILD_CONFIGURATIONS.RELEASE.toLowerCase();
    }
    getMobileProvisionData(provisionPath) {
        const provisionText = this.$fs.readText(path.resolve(provisionPath));
        const provisionPlistText = provisionText.substring(provisionText.indexOf(constants.CRYPTO.PLIST_HEADER), provisionText.indexOf(constants.CRYPTO.PLIST_FOOTER) + constants.CRYPTO.PLIST_FOOTER.length);
        return plist.parse(provisionPlistText);
    }
    zipProject(projectDir) {
        return __awaiter(this, void 0, void 0, function* () {
            let tempDir = path.join(projectDir, constants.CLOUD_TEMP_DIR_NAME);
            this.$fs.ensureDirectoryExists(tempDir);
            let projectZipFile = path.join(tempDir, "Build.zip");
            this.$fs.deleteFile(projectZipFile);
            let files = this.getProjectFiles(projectDir, ["node_modules", "platforms", constants.CLOUD_TEMP_DIR_NAME, "**/.*"]);
            yield this.$fs.zipFiles(projectZipFile, files, p => this.getProjectRelativePath(p, projectDir));
            return projectZipFile;
        });
    }
    getProjectFiles(projectFilesPath, excludedProjectDirsAndFiles, filter, opts) {
        const projectFiles = this.$fs.enumerateFilesInDirectorySync(projectFilesPath, (filePath, stat) => {
            const isFileExcluded = this.isFileExcluded(path.relative(projectFilesPath, filePath), excludedProjectDirsAndFiles);
            const isFileFiltered = filter ? filter(filePath, stat) : false;
            return !isFileExcluded && !isFileFiltered;
        }, opts);
        this.$logger.trace("getProjectFiles for cloud build: ", projectFiles);
        return projectFiles;
    }
    isFileExcluded(filePath, excludedProjectDirsAndFiles) {
        return !!_.find(excludedProjectDirsAndFiles, (pattern) => minimatch(filePath, pattern, { nocase: true }));
    }
    getProjectRelativePath(fullPath, projectDir) {
        projectDir = path.join(projectDir, path.sep);
        if (!_.startsWith(fullPath, projectDir)) {
            throw new Error("File is not part of the project.");
        }
        return fullPath.substring(projectDir.length);
    }
}
exports.CloudBuildHelper = CloudBuildHelper;
$injector.register("nsCloudBuildHelper", CloudBuildHelper);
