"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
class CloudConfigManager {
    constructor($fs, $options) {
        this.$fs = $fs;
        this.$options = $options;
        const baseConfigPath = this.getConfigPath(CloudConfigManager.CONFIG_FILE_NAME);
        let serverConfig = null;
        if (this.$fs.exists(baseConfigPath)) {
            serverConfig = this.getCurrentConfigData();
        }
        else {
            serverConfig = this.loadConfig(CloudConfigManager.BASE_CONFIG_FILE_NAME);
        }
        this.mergeConfig(this, serverConfig);
    }
    reset() {
        this.applyConfig("production");
    }
    applyConfig(configName, options, globalServerConfig) {
        const baseConfig = this.loadConfig(CloudConfigManager.BASE_CONFIG_FILE_NAME);
        const newConfig = this.loadConfig(`${CloudConfigManager.CONFIG_FILE_NAME}-${configName}`, options);
        this.mergeConfig(baseConfig, newConfig, globalServerConfig);
        this.saveConfig(baseConfig, CloudConfigManager.CONFIG_FILE_NAME);
    }
    printConfigData() {
        const config = this.getCurrentConfigData();
        console.log(JSON.stringify(config, null, 2));
    }
    getCurrentConfigData() {
        return this.loadConfig(CloudConfigManager.CONFIG_FILE_NAME);
    }
    getServiceDomainName(serviceName) {
        const config = this.getCurrentConfigData();
        const serviceConfig = config.cloudServices[serviceName];
        if (serviceConfig.fullHostName) {
            return serviceConfig.fullHostName;
        }
        return `${serviceConfig.subdomain}.${config.domainName}`;
    }
    getServiceValueOrDefault(serviceName, valueName) {
        const config = this.getCurrentConfigData();
        const serviceConfig = config.cloudServices[serviceName];
        if (_.has(serviceConfig, valueName)) {
            return serviceConfig[valueName];
        }
        return config[valueName];
    }
    getCloudServicesDomainNames() {
        const cloudServices = this.getCurrentConfigData().cloudServices;
        return _(cloudServices)
            .keys()
            .map(s => this.getServiceDomainName(s))
            .value();
    }
    loadConfig(name, options) {
        const configFileName = this.getConfigPath(name, options);
        return this.$fs.readJson(configFileName);
    }
    getConfigPath(filename, options) {
        if (options && options.configurationFile) {
            return options.configurationFile;
        }
        const dirname = path.join(__dirname, "../server-configs/");
        return path.join(dirname, filename + ".json");
    }
    saveConfig(config, name) {
        const configNoFunctions = Object.create(null);
        _.each(config, (entry, key) => {
            if (typeof entry !== "function") {
                configNoFunctions[key] = entry;
            }
        });
        const configFileName = this.getConfigPath(name);
        return this.$fs.writeJson(configFileName, configNoFunctions);
    }
    mergeConfig(config, mergeFrom, globalServerConfig) {
        _.extend(config, mergeFrom);
        if (globalServerConfig) {
            const filteredGlobalConfig = _.omitBy(globalServerConfig, _.isUndefined);
            _.extend(config, filteredGlobalConfig);
        }
    }
}
CloudConfigManager.CONFIG_FILE_NAME = "config";
CloudConfigManager.BASE_CONFIG_FILE_NAME = `${CloudConfigManager.CONFIG_FILE_NAME}-base`;
exports.CloudConfigManager = CloudConfigManager;
$injector.register("nsCloudConfigManager", CloudConfigManager);
