"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const constants_1 = require("../../constants");
class CommunicationChannelBase extends events_1.EventEmitter {
    constructor(cloudOperationId, data, $logger) {
        super();
        this.cloudOperationId = cloudOperationId;
        this.data = data;
        this.$logger = $logger;
        this.echoSequenceNumber = 0;
        this.echoReplySequenceNumber = 0;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected) {
                const connect = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    this.connectTimeout = setTimeout(() => {
                        clearTimeout(this.connectTimeout);
                        reject(new Error("Failed to connect to the communication channel."));
                    }, CommunicationChannelBase.CONNECT_TIMEOUT);
                    try {
                        yield this.connectCore();
                        yield this.handshake();
                        this.isConnected = true;
                    }
                    catch (err) {
                        clearTimeout(this.connectTimeout);
                        reject(err);
                        return;
                    }
                    clearTimeout(this.connectTimeout);
                    resolve();
                }));
                yield connect;
                this.startPing();
            }
        });
    }
    sendMessage(message) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected) {
                const result = yield this.sendMessageCore(message);
                return result;
            }
            throw new Error("Communication channel not connected");
        });
    }
    close(code, reason) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected) {
                return;
            }
            try {
                clearTimeout(this.connectTimeout);
                if (this.pingInterval) {
                    clearInterval(this.pingInterval);
                    this.pingInterval = null;
                }
                const stopMsg = {
                    type: constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_STOP,
                    cloudOperationId: this.cloudOperationId,
                    body: {
                        code,
                        reason
                    }
                };
                yield this.sendMessage(stopMsg);
                yield this.closeCore(code, reason);
            }
            catch (err) {
                this.$logger.trace(err);
            }
            this.isConnected = false;
            this.emit(constants_1.CloudCommunicationEvents.CLOSE, code, reason);
        });
    }
    handleMessage(m) {
        return __awaiter(this, void 0, void 0, function* () {
            let msg;
            if (_.isBuffer(m)) {
                msg = JSON.parse(m.toString("utf8"));
            }
            else if (_.isString(m)) {
                msg = JSON.parse(m);
            }
            else {
                msg = m;
            }
            if (msg.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_SERVER_HELLO) {
                this.handshakeResolve();
            }
            if (msg.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_HANDSHAKE_ERROR) {
                const body = msg.body;
                this.handshakeReject(new Error(body.message));
                return;
            }
            else if (msg.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_ECHO) {
                const body = msg.body;
                const echoReply = {
                    type: constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_ECHO_REPLY,
                    cloudOperationId: this.cloudOperationId,
                    body
                };
                yield this.sendMessage(echoReply);
            }
            else if (msg.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_ECHO_REPLY) {
                const body = msg.body;
                if (body.identifier === this.cloudOperationId) {
                    this.echoReplySequenceNumber = body.sequenceNumber;
                }
            }
            this.emit(constants_1.CloudCommunicationEvents.MESSAGE, msg);
            return;
        });
    }
    handshake() {
        return __awaiter(this, void 0, void 0, function* () {
            const handshakePromise = new Promise((resolve, reject) => {
                this.handshakeResolve = resolve;
                this.handshakeReject = reject;
            });
            const clientHello = {
                type: constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_CLIENT_HELLO,
                cloudOperationId: this.cloudOperationId,
                body: {
                    communicationProtocolVersion: CommunicationChannelBase.COMMUNICATION_PROTOCOL_VERSION
                }
            };
            yield this.sendMessageCore(clientHello);
            yield handshakePromise;
        });
    }
    startPing() {
        this.pingInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
            if (this.echoSequenceNumber - this.echoReplySequenceNumber >= CommunicationChannelBase.MISSING_PING_REPLIES_COUNT) {
                return this.close(constants_1.CloudCommunicationChannelExitCodes.MISSING_ECHO_REPLIES, "Communication channel did not receive echo replies.");
            }
            const echoMsg = {
                type: constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_ECHO,
                cloudOperationId: this.cloudOperationId,
                body: {
                    identifier: this.cloudOperationId,
                    sequenceNumber: this.echoSequenceNumber++
                }
            };
            yield this.sendMessage(echoMsg);
        }), CommunicationChannelBase.PING_INTERVAL);
    }
}
CommunicationChannelBase.COMMUNICATION_PROTOCOL_VERSION = "v2";
CommunicationChannelBase.PING_INTERVAL = 10000;
CommunicationChannelBase.MISSING_PING_REPLIES_COUNT = 6;
CommunicationChannelBase.CONNECT_TIMEOUT = 60000;
exports.CommunicationChannelBase = CommunicationChannelBase;
