"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const semver = require("semver");
const constants_1 = require("../constants");
const helpers_1 = require("../helpers");
const helpers_2 = require("../helpers");
class BuildCommandHelper {
    constructor($nsCloudBuildService, $nsCloudErrorsService, $logger, $prompter, $mobileHelper, $projectData, $injector, $options, $fs, $staticConfig) {
        this.$nsCloudBuildService = $nsCloudBuildService;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$mobileHelper = $mobileHelper;
        this.$projectData = $projectData;
        this.$injector = $injector;
        this.$options = $options;
        this.$fs = $fs;
        this.$staticConfig = $staticConfig;
        this.shouldUseOldLocalBuildService = true;
        this.$projectData.initializeProjectData();
        const cliVersion = this.$staticConfig.version;
        this.shouldUseOldLocalBuildService = semver.valid(cliVersion) && semver.lt(cliVersion, semver.prerelease(cliVersion) ? "5.4.0-2019-05-16-13277" : "6.0.0");
    }
    get $localBuildService() {
        if (this.shouldUseOldLocalBuildService) {
            return this.$injector.resolve("localBuildService");
        }
        return this.$injector.resolve("buildController");
    }
    buildPlatform(platform, buildConfig, projectData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const buildData = this.getCloudBuildData(platform);
            buildData.iOSBuildData.buildForDevice = buildConfig.buildForDevice;
            const buildResultData = yield this.$nsCloudBuildService.build(buildData.projectSettings, buildData.platform, buildData.buildConfiguration, this.$options.accountId, buildData.androidBuildData, buildData.iOSBuildData);
            return buildResultData.outputFilePath;
        });
    }
    getCloudBuildData(platformArg) {
        global.showErrorForStoppedCloudBuilds();
        const platform = this.$mobileHelper.validatePlatformName(platformArg);
        this.$logger.info(`Executing cloud build with platform: ${platform}.`);
        const nativescriptData = this.$fs.readJson(path.join(this.$projectData.projectDir, "package.json")).nativescript;
        let pathToCertificate = "";
        if (this.$mobileHelper.isAndroidPlatform(platform)) {
            pathToCertificate = this.$options.keyStorePath ? path.resolve(this.$options.keyStorePath) : "";
        }
        else if (this.$mobileHelper.isiOSPlatform(platform)) {
            pathToCertificate = this.$options.certificate ? path.resolve(this.$options.certificate) : "";
        }
        else {
            this.$nsCloudErrorsService.fail(`Currently only ${this.$mobileHelper.platformNames.join(' ')} platforms are supported.`);
        }
        const pathToProvision = this.$options.provision ? path.resolve(this.$options.provision) : "";
        const projectId = helpers_1.getProjectId(this.$projectData, platform.toLowerCase());
        const projectSettings = {
            nativescriptData,
            projectDir: this.$projectData.projectDir,
            projectId,
            projectName: this.$projectData.projectName,
            bundle: !!this.$options.bundle,
            sharedCloud: this.$options.sharedCloud,
            flavorId: this.$options.vmTemplateName,
            workflowName: this.$options.workflow && this.$options.workflow.name,
            workflowUrl: this.$options.workflow && this.$options.workflow.url,
            clean: this.$options.clean,
            env: this.$options.env,
            useHotModuleReload: this.$options.hmr
        };
        const buildConfiguration = this.$options.release ? constants_1.CLOUD_BUILD_CONFIGURATIONS.RELEASE : constants_1.CLOUD_BUILD_CONFIGURATIONS.DEBUG;
        return {
            projectSettings,
            platform,
            buildConfiguration,
            androidBuildData: {
                pathToCertificate,
                certificatePassword: this.$options.keyStorePassword,
                aab: this.$options.aab
            },
            iOSBuildData: {
                pathToCertificate,
                certificatePassword: this.$options.certificatePassword,
                pathToProvision,
                buildForDevice: !this.$options.emulator
            }
        };
    }
    getAppleCredentials(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            let { username, password } = this.getUsernameAndPasswordFromArgs(args);
            if (!username) {
                username = yield this.$prompter.getString("Apple ID", { allowEmpty: false });
            }
            if (!password) {
                password = yield this.$prompter.getPassword("Apple ID password");
            }
            return {
                username,
                password
            };
        });
    }
    getExtendedAppleCredentials(args, options) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const extendedCredentials = {
                appleApplicationSpecificPassword: options.appleApplicationSpecificPassword,
                appleSession: options.appleSessionBase64 ? Buffer.from(options.appleSessionBase64, "base64").toString() : undefined
            };
            let credentials;
            if (!options.appleSessionBase64) {
                credentials = yield this.getAppleCredentials(args);
            }
            else {
                credentials = this.getUsernameAndPasswordFromArgs(args);
                if (!helpers_2.isInteractive() && (!credentials.username || !credentials.password)) {
                    this.$nsCloudErrorsService.fail("Please provide Apple ID and Apple ID password");
                }
            }
            return _.merge(credentials, extendedCredentials);
        });
    }
    buildForPublishingPlatform(platformArg) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            let packagePath;
            const platform = this.$mobileHelper.validatePlatformName(platformArg);
            if (this.$options.local) {
                packagePath = yield this.executeLocalBuild(platform);
            }
            else {
                const buildData = this.getCloudBuildData(platform);
                buildData.buildConfiguration = constants_1.CLOUD_BUILD_CONFIGURATIONS.RELEASE;
                const cloudResult = yield this.$nsCloudBuildService.build(buildData.projectSettings, buildData.platform, buildData.buildConfiguration, this.$options.accountId, buildData.androidBuildData, buildData.iOSBuildData, { shouldPrepare: true });
                packagePath = cloudResult.qrData ? cloudResult.qrData.originalUrl : cloudResult.outputFilePath;
            }
            return packagePath;
        });
    }
    getUsernameAndPasswordFromArgs(args) {
        return { username: args[0], password: args[1] };
    }
    executeLocalBuild(platform) {
        return __awaiter(this, void 0, void 0, function* () {
            let packagePath = null;
            if (this.shouldUseOldLocalBuildService) {
                packagePath = yield this.$localBuildService.build(platform, {
                    release: true,
                    buildForDevice: true,
                    clean: this.$options.clean,
                    teamId: this.$options.teamId,
                    bundle: !!this.$options.bundle,
                    device: this.$options.device,
                    projectDir: this.$options.path,
                    provision: this.$options.provision,
                    keyStoreAlias: this.$options.keyStoreAlias,
                    keyStoreAliasPassword: this.$options.keyStoreAliasPassword,
                    keyStorePassword: this.$options.keyStorePassword,
                    keyStorePath: this.$options.keyStorePath,
                    useHotModuleReload: this.$options.hmr,
                    env: this.$options.env,
                    iCloudContainerEnvironment: this.$options.iCloudContainerEnvironment
                }, this.$options.platformTemplate);
            }
            else {
                const $buildDataService = this.$injector.resolve("buildDataService");
                const buildData = $buildDataService.getBuildData(this.$projectData.projectDir, platform, this.$options);
                packagePath = yield this.$localBuildService.build(buildData);
            }
            return packagePath;
        });
    }
}
exports.BuildCommandHelper = BuildCommandHelper;
$injector.register("nsCloudBuildCommandHelper", BuildCommandHelper);
