"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const helpers_1 = require("../helpers");
const constants_1 = require("../constants");
class AuthenticationService {
    constructor($fs, $injector, $logger, $nsCloudHttpServer, $nsCloudServerAuthService, $nsCloudServicesPolicyService, $nsCloudUserService, $opener) {
        this.$fs = $fs;
        this.$injector = $injector;
        this.$logger = $logger;
        this.$nsCloudHttpServer = $nsCloudHttpServer;
        this.$nsCloudServerAuthService = $nsCloudServerAuthService;
        this.$nsCloudServicesPolicyService = $nsCloudServicesPolicyService;
        this.$nsCloudUserService = $nsCloudUserService;
        this.$opener = $opener;
    }
    get $nsCloudAccountsService() {
        return this.$injector.resolve("nsCloudAccountsService");
    }
    login(options) {
        return __awaiter(this, void 0, void 0, function* () {
            options = options || {};
            if (this.localhostServer) {
                this.cancelLogin();
            }
            let timeoutID = undefined;
            let authCompleteResolveAction;
            let isResolved = false;
            this.$logger.info("Launching login page in browser.");
            let loginUrl;
            this.localhostServer = this.$nsCloudHttpServer.createServer({
                routes: {
                    "/": (request, response) => {
                        this.$logger.debug("Login complete: " + request.url);
                        const parsedUrl = url_1.parse(request.url, true);
                        const serverResponse = parsedUrl.query.response;
                        if (serverResponse) {
                            response.statusCode = constants_1.HTTP_STATUS_CODES.FOUND;
                            response.setHeader(constants_1.HTTP_HEADERS.LOCATION, parsedUrl.query.loginCompleteUrl);
                            this.killLocalhostServer();
                            isResolved = true;
                            const decodedResponse = Buffer.from(serverResponse, "base64").toString();
                            this.rejectLoginPromiseAction = null;
                            authCompleteResolveAction(decodedResponse);
                            response.end();
                        }
                        else {
                            this.$nsCloudHttpServer.redirect(response, loginUrl);
                        }
                    }
                }
            });
            this.localhostServer.listen(0);
            yield this.$fs.futureFromEvent(this.localhostServer, "listening");
            const authComplete = new Promise((resolve, reject) => {
                authCompleteResolveAction = resolve;
                this.rejectLoginPromiseAction = reject;
                const port = this.localhostServer.address().port;
                loginUrl = this.$nsCloudServerAuthService.getLoginUrl(port);
                this.$logger.debug("Login URL is '%s'", loginUrl);
                if (!options.openAction) {
                    this.$opener.open(loginUrl);
                }
                else {
                    options.openAction(loginUrl);
                }
                if (!helpers_1.isInteractive()) {
                    const timeout = options.hasOwnProperty("timeout")
                        ? +options.timeout
                        : AuthenticationService.DEFAULT_NONINTERACTIVE_LOGIN_TIMEOUT_MS;
                    if (timeout > 0) {
                        timeoutID = setTimeout(() => {
                            if (!isResolved) {
                                this.$logger.debug("Aborting login procedure due to inactivity.");
                                reject(new Error("Aborting login procedure due to inactivity."));
                            }
                        }, timeout);
                    }
                }
            });
            const loginResponse = yield authComplete;
            if (timeoutID !== undefined) {
                clearTimeout(timeoutID);
            }
            const userData = JSON.parse(loginResponse);
            this.$nsCloudUserService.setUserData(userData);
            const userInfo = userData.userInfo;
            yield this.$nsCloudServicesPolicyService.acceptCloudServicesPolicy();
            yield this.$nsCloudAccountsService.sendPoliciesToCloud();
            return userInfo;
        });
    }
    devLogin(username, password, instanceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userData = yield this.$nsCloudServerAuthService.devLogin(username, password, instanceId);
            userData.instanceId = instanceId;
            this.$nsCloudUserService.setUserData(userData);
            return userData.userInfo;
        });
    }
    cancelLogin() {
        this.$logger.trace("Cancel login.");
        if (this.localhostServer) {
            this.$logger.trace("Stopping localhost server.");
            this.killLocalhostServer();
        }
        if (this.rejectLoginPromiseAction) {
            this.$logger.trace("Rejecting login promise.");
            this.rejectLoginPromiseAction(new Error("Login canceled."));
            this.rejectLoginPromiseAction = null;
        }
    }
    logout(options) {
        options = options || {};
        const logoutUrl = this.$nsCloudServerAuthService.getLogoutUrl();
        this.$logger.trace(`Logging out. Logout url is: ${logoutUrl}`);
        if (options.openAction) {
            options.openAction(logoutUrl);
        }
        else {
            this.$opener.open(logoutUrl);
        }
        this.$nsCloudUserService.clearUserData();
    }
    refreshCurrentUserToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const userData = this.$nsCloudUserService.getUserData();
            const token = yield this.$nsCloudServerAuthService.refreshToken(userData.refreshToken);
            this.$nsCloudUserService.setToken(token);
        });
    }
    isUserLoggedIn() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$nsCloudUserService.hasUser()) {
                const tokenState = yield this.getCurrentUserTokenState();
                if (tokenState.isTokenValid) {
                    return true;
                }
                try {
                    this.$logger.trace("The access token of the user has expired. Trying to issue new one.");
                    yield this.refreshCurrentUserToken();
                    return true;
                }
                catch (err) {
                    this.$logger.trace("Error while trying to issue new access token:");
                    this.$logger.trace(err);
                }
            }
            return false;
        });
    }
    getCurrentUserTokenState() {
        return __awaiter(this, void 0, void 0, function* () {
            const userData = this.$nsCloudUserService.getUserData();
            const tokenState = yield this.$nsCloudServerAuthService.getTokenState(userData.accessToken);
            return tokenState;
        });
    }
    killLocalhostServer() {
        this.localhostServer.close();
        this.localhostServer = null;
    }
}
AuthenticationService.DEFAULT_NONINTERACTIVE_LOGIN_TIMEOUT_MS = 15 * 60 * 1000;
exports.AuthenticationService = AuthenticationService;
$injector.register("nsCloudAuthenticationService", AuthenticationService);
