"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants = require("../constants");
const os_1 = require("os");
class CloudBuildPropertiesService {
    constructor($nsCloudBuildHelper, $nsCloudErrorsService, $fs, $mobileHelper) {
        this.$nsCloudBuildHelper = $nsCloudBuildHelper;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$fs = $fs;
        this.$mobileHelper = $mobileHelper;
    }
    validateBuildProperties(platform, buildConfiguration, appId, androidBuildData, iOSBuildData) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.$mobileHelper.isAndroidPlatform(platform) && this.$nsCloudBuildHelper.isReleaseConfiguration(buildConfiguration)) {
                if (!androidBuildData || !androidBuildData.pathToCertificate) {
                    this.$nsCloudErrorsService.fail("When building for Release configuration, you must specify valid Certificate and its password.");
                }
                if (!this.$fs.exists(androidBuildData.pathToCertificate)) {
                    this.$nsCloudErrorsService.fail(`The specified certificate: ${androidBuildData.pathToCertificate} does not exist. Verify the location is correct.`);
                }
                if (!androidBuildData.certificatePassword) {
                    this.$nsCloudErrorsService.fail(`No password specified for certificate ${androidBuildData.pathToCertificate}.`);
                }
                if (androidBuildData.certificatePassword.length < 6) {
                    this.$nsCloudErrorsService.fail("The password for Android certificate must be at least 6 characters long.");
                }
            }
            else if (this.$mobileHelper.isiOSPlatform(platform) && iOSBuildData.buildForDevice) {
                if (!iOSBuildData || !iOSBuildData.pathToCertificate || !iOSBuildData.certificatePassword || !iOSBuildData.pathToProvision) {
                    this.$nsCloudErrorsService.fail("When building for iOS you must specify valid Mobile Provision, Certificate and its password.");
                }
                if (!this.$fs.exists(iOSBuildData.pathToCertificate)) {
                    this.$nsCloudErrorsService.fail(`The specified certificate: ${iOSBuildData.pathToCertificate} does not exist. Verify the location is correct.`);
                }
                if (!this.$fs.exists(iOSBuildData.pathToProvision)) {
                    this.$nsCloudErrorsService.fail(`The specified provision: ${iOSBuildData.pathToProvision} does not exist. Verify the location is correct.`);
                }
                const certInfo = this.$nsCloudBuildHelper.getCertificateInfo(iOSBuildData.pathToCertificate, iOSBuildData.certificatePassword);
                const certBase64 = this.getCertificateBase64(certInfo.pemCert);
                const provisionData = this.$nsCloudBuildHelper.getMobileProvisionData(iOSBuildData.pathToProvision);
                const provisionCertificatesBase64 = _.map(provisionData.DeveloperCertificates, c => c.toString('base64'));
                const now = Date.now();
                let provisionAppId = provisionData.Entitlements['application-identifier'];
                _.each(provisionData.ApplicationIdentifierPrefix, prefix => {
                    provisionAppId = provisionAppId.replace(`${prefix}.`, "");
                });
                let errors = [];
                if (provisionAppId !== "*") {
                    let provisionIdentifierPattern = new RegExp(this.getRegexPattern(provisionAppId));
                    if (!provisionIdentifierPattern.test(appId)) {
                        errors.push(`The specified provision's (${iOSBuildData.pathToProvision}) application identifier (${provisionAppId}) doesn't match your project's application identifier (${appId}).`);
                    }
                }
                if (certInfo.organization !== constants.APPLE_INC) {
                    errors.push(`The specified certificate: ${iOSBuildData.pathToCertificate} is issued by an untrusted organization. Please provide a certificate issued by ${constants.APPLE_INC}`);
                }
                if (now > provisionData.ExpirationDate.getTime()) {
                    errors.push(`The specified provision: ${iOSBuildData.pathToProvision} has expired.`);
                }
                if (now < certInfo.validity.notBefore.getTime() || now > certInfo.validity.notAfter.getTime()) {
                    errors.push(`The specified certificate: ${iOSBuildData.pathToCertificate} has expired.`);
                }
                if (!_.includes(provisionCertificatesBase64, certBase64)) {
                    errors.push(`The specified provision: ${iOSBuildData.pathToProvision} does not include the specified certificate: ${iOSBuildData.pathToCertificate}. Please specify a different provision or certificate.`);
                }
                if (iOSBuildData.deviceIdentifier && !_.includes(provisionData.ProvisionedDevices, iOSBuildData.deviceIdentifier)) {
                    errors.push(`The specified provision: ${iOSBuildData.pathToProvision} does not include the specified device: ${iOSBuildData.deviceIdentifier}.`);
                }
                if (errors.length) {
                    this.$nsCloudErrorsService.fail(errors.join(os_1.EOL));
                }
            }
        });
    }
    getAndroidBuildProperties(projectSettings, buildProps, amazonStorageEntriesData, androidBuildData) {
        return __awaiter(this, void 0, void 0, function* () {
            const buildConfiguration = buildProps.properties.buildConfiguration;
            if (this.$nsCloudBuildHelper.isReleaseConfiguration(buildConfiguration)) {
                const certificateData = _.find(amazonStorageEntriesData, amazonStorageEntryData => amazonStorageEntryData.filePath === androidBuildData.pathToCertificate);
                buildProps.properties.keyStoreName = certificateData.fileName;
                buildProps.properties.keyStoreAlias = this.$nsCloudBuildHelper.getCertificateInfo(androidBuildData.pathToCertificate, androidBuildData.certificatePassword).friendlyName;
                buildProps.properties.keyStorePassword = androidBuildData.certificatePassword;
                buildProps.properties.keyStoreAliasPassword = androidBuildData.certificatePassword;
                buildProps.buildFiles.push({
                    disposition: certificateData.disposition,
                    sourceUri: certificateData.publicDownloadUrl
                });
            }
            return buildProps;
        });
    }
    getiOSBuildProperties(projectSettings, buildProps, amazonStorageEntriesData, iOSBuildData) {
        return __awaiter(this, void 0, void 0, function* () {
            if (iOSBuildData.buildForDevice) {
                const provisionData = this.$nsCloudBuildHelper.getMobileProvisionData(iOSBuildData.pathToProvision);
                const certificateData = _.find(amazonStorageEntriesData, amazonStorageEntryData => amazonStorageEntryData.filePath === iOSBuildData.pathToCertificate);
                const provisonData = _.find(amazonStorageEntriesData, amazonStorageEntryData => amazonStorageEntryData.filePath === iOSBuildData.pathToProvision);
                buildProps.buildFiles.push({
                    sourceUri: certificateData.publicDownloadUrl,
                    disposition: certificateData.disposition
                }, {
                    sourceUri: provisonData.publicDownloadUrl,
                    disposition: provisonData.disposition
                });
                buildProps.properties.certificatePassword = iOSBuildData.certificatePassword;
                buildProps.properties.codeSigningIdentity = this.$nsCloudBuildHelper.getCertificateInfo(iOSBuildData.pathToCertificate, iOSBuildData.certificatePassword).commonName;
                const cloudProvisionsData = [{
                        suffixId: "",
                        templateName: "PROVISION_",
                        identifier: provisionData.UUID,
                        isDefault: true,
                        fileName: provisonData.fileName,
                        appGroups: [],
                        provisionType: this.$nsCloudBuildHelper.getProvisionType(provisionData),
                        name: provisionData.Name
                    }];
                buildProps.properties.mobileProvisionIdentifiers = JSON.stringify(cloudProvisionsData);
                buildProps.properties.defaultMobileProvisionIdentifier = provisionData.UUID;
            }
            else {
                buildProps.properties.simulator = true;
            }
            return buildProps;
        });
    }
    getCertificateBase64(cert) {
        return cert.replace(/\s/g, "").substr(constants.CRYPTO.CERTIFICATE_HEADER.length).slice(0, -constants.CRYPTO.CERTIFICATE_FOOTER.length);
    }
    getRegexPattern(appIdentifier) {
        let starPlaceholder = "<!StarPlaceholder!>";
        let escapedIdentifier = this.escape(this.stringReplaceAll(appIdentifier, "*", starPlaceholder));
        let replacedIdentifier = this.stringReplaceAll(escapedIdentifier, starPlaceholder, ".*");
        return "^" + replacedIdentifier + "$";
    }
    escape(s) {
        return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
    }
    stringReplaceAll(inputString, find, replace) {
        return inputString.split(find).join(replace);
    }
}
exports.CloudBuildPropertiesService = CloudBuildPropertiesService;
$injector.register("nsCloudBuildPropertiesService", CloudBuildPropertiesService);
