"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const os_1 = require("os");
const cloud_service_1 = require("./cloud-service");
class CloudProjectService extends cloud_service_1.CloudService {
    constructor($constants, $nsCloudErrorsService, $fs, $httpClient, $logger, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService, $nsCloudServerProjectService, $projectHelper) {
        super($nsCloudErrorsService, $fs, $httpClient, $logger, $constants, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService);
        this.$nsCloudServerProjectService = $nsCloudServerProjectService;
        this.$projectHelper = $projectHelper;
    }
    get failedError() {
        return "Cloud cleanup failed.";
    }
    get failedToStartError() {
        return "Failed to start cloud cleanup.";
    }
    getServerOperationOutputDirectory(options) {
        return path.join(options.projectDir, "cleanup-results");
    }
    cleanupProject(cleanupRequestData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            let identifiers;
            const cleanupTaskResults = [];
            if (typeof cleanupRequestData.appIdentifier === "string") {
                identifiers = [cleanupRequestData.appIdentifier];
            }
            else {
                identifiers = _.uniq(_.values(cleanupRequestData.appIdentifier));
            }
            for (let index = 0; index < identifiers.length; index++) {
                const appIdentifier = identifiers[index];
                const taskResult = yield this.startCleanProject({ appIdentifier, projectName: cleanupRequestData.projectName });
                cleanupTaskResults.push(taskResult);
            }
            return {
                cleanupTaskResults
            };
        });
    }
    startCleanProject(cleanupProjectData) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.executeCloudOperation("Cloud cleanup", (cloudOperationId) => __awaiter(this, void 0, void 0, function* () {
                const res = yield this.executeCleanupProject(cloudOperationId, cleanupProjectData);
                this.$logger.trace(`Cleanup [${cloudOperationId}] result: `, res);
                return res;
            }));
            return result;
        });
    }
    executeCleanupProject(cloudOperationId, { appIdentifier, projectName }) {
        return __awaiter(this, void 0, void 0, function* () {
            this.$logger.info(`Cloud cleanup: Application Id: ${appIdentifier}, Project Name: ${projectName}.`);
            const sanitizedProjectName = this.$projectHelper.sanitizeName(projectName);
            const cleanupProjectData = {
                appIdentifier: appIdentifier,
                projectName: sanitizedProjectName,
                templateAppName: sanitizedProjectName
            };
            const cleanupResponse = yield this.$nsCloudServerProjectService.cleanupProjectData(cleanupProjectData);
            this.$logger.trace("Cleanup response: ", cleanupResponse);
            if (cleanupResponse.warnings && cleanupResponse.warnings.length) {
                this.$logger.error("### Errors while cleaning cloud project data:");
                this.$logger.error(_.map(cleanupResponse.warnings, w => w.message).join(os_1.EOL));
            }
            this.$logger.info(`### AWS CodeCommit cleanup${os_1.EOL}`);
            if (cleanupResponse.codeCommitResponse) {
                this.$logger.info(`Cleaned repository: ${cleanupResponse.codeCommitResponse.repositoryId}.`);
            }
            else {
                this.$logger.info("No AWS CodeCommit data to clean.");
            }
            this.$logger.info(`${os_1.EOL}### Build machines cleanup${os_1.EOL}`);
            const tasksResults = yield this.waitForAllCleanOperationsToFinish(cleanupResponse.buildMachineResponse);
            const result = {
                cloudOperationId,
                warnings: cleanupResponse.warnings,
                codeCommitResponse: cleanupResponse.codeCommitResponse,
                cloudTasksResults: tasksResults
            };
            return result;
        });
    }
    waitForAllCleanOperationsToFinish(serverResponses) {
        return __awaiter(this, void 0, void 0, function* () {
            const childTasks = _.map(serverResponses, task => this.waitForChildTaskToFinish(task));
            const tasksResults = {};
            for (let task of childTasks) {
                const result = yield task;
                if (result) {
                    tasksResults[result.cloudOperationId] = result.taskResult;
                }
            }
            return tasksResults;
        });
    }
    waitForChildTaskToFinish(task) {
        return __awaiter(this, void 0, void 0, function* () {
            const childCloudOperationId = task.cloudOperationId;
            let output = `Child cloud operation id: ${childCloudOperationId}`;
            const taskResult = yield this.waitForCloudOperationToFinish(childCloudOperationId, task, { silent: true, hideBuildMachineMetadata: true });
            output += yield this.getCollectedLogs(task.cloudOperationId);
            this.$logger.info(output);
            if (taskResult.code !== 0) {
                this.$logger.error(`${childCloudOperationId} error: ${taskResult.errors}`);
            }
            return { taskResult, cloudOperationId: task.cloudOperationId };
        });
    }
}
exports.CloudProjectService = CloudProjectService;
$injector.register("nsCloudProjectService", CloudProjectService);
