"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const os_1 = require("os");
const constants_1 = require("../constants");
const cloud_service_1 = require("./cloud-service");
const helpers_1 = require("../helpers");
class CloudPublishService extends cloud_service_1.CloudService {
    constructor($constants, $nsCloudErrorsService, $fs, $httpClient, $logger, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService, $nsCloudServerBuildService, $devicePlatformsConstants, $nsCloudUploadService, $projectDataService) {
        super($nsCloudErrorsService, $fs, $httpClient, $logger, $constants, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService);
        this.$nsCloudServerBuildService = $nsCloudServerBuildService;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$nsCloudUploadService = $nsCloudUploadService;
        this.$projectDataService = $projectDataService;
    }
    get failedError() {
        return "Publishing failed.";
    }
    get failedToStartError() {
        return "Failed to start publishing.";
    }
    publishToItunesConnect(publishData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            yield this.executeCloudOperation("Cloud publish iOS", (cloudOperationId) => __awaiter(this, void 0, void 0, function* () {
                this.validatePublishData(publishData);
                const projectData = this.$projectDataService.getProjectData(publishData.projectDir);
                const appIdentifier = helpers_1.getProjectId(projectData, this.$devicePlatformsConstants.iOS.toLowerCase());
                const publishRequestData = {
                    cloudOperationId,
                    appIdentifier,
                    credentials: publishData.credentials,
                    packagePaths: publishData.packagePaths,
                    platform: this.$devicePlatformsConstants.iOS
                };
                yield this.publishCore(publishRequestData, publishData, this.getiOSError.bind(this));
            }));
        });
    }
    publishToGooglePlay(publishData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            yield this.executeCloudOperation("Cloud publish Android", (cloudOperationId) => __awaiter(this, void 0, void 0, function* () {
                this.validatePublishData(publishData);
                if (!publishData.pathToAuthJson || !this.$fs.exists(publishData.pathToAuthJson)) {
                    this.$nsCloudErrorsService.fail("Cannot perform publish - auth json file is not supplied or the provided path does not exist.");
                }
                let authJson;
                try {
                    authJson = JSON.stringify(this.$fs.readJson(publishData.pathToAuthJson));
                }
                catch (ex) {
                    this.$nsCloudErrorsService.fail("Cannot perform publish - auth json file is not in JSON format.");
                }
                publishData.track = publishData.track || constants_1.DEFAULT_ANDROID_PUBLISH_TRACK;
                const projectData = this.$projectDataService.getProjectData(publishData.projectDir);
                const appIdentifier = helpers_1.getProjectId(projectData, this.$devicePlatformsConstants.Android.toLowerCase());
                yield this.publishCore({
                    cloudOperationId,
                    appIdentifier,
                    credentials: {
                        authJson
                    },
                    packagePaths: publishData.packagePaths,
                    platform: this.$devicePlatformsConstants.Android,
                    track: publishData.track,
                    androidReleaseStatus: publishData.androidReleaseStatus,
                }, publishData, this.getAndroidError.bind(this));
            }));
        });
    }
    getiOSError(publishResult, publishRequestData) {
        const itmsMessage = this.getFormattedError(publishResult, CloudPublishService.ITMS_ERROR_REGEX);
        const err = new Error(`${publishResult.errors}${os_1.EOL}${itmsMessage}`);
        return err;
    }
    getAndroidError(publishResult, publishRequestData) {
        const generalMessage = this.getFormattedError(publishResult, CloudPublishService.GENERAL_ERROR_REGEX);
        return new Error(`${publishResult.errors}${os_1.EOL}${generalMessage}`);
    }
    publishCore(publishRequestData, publishDataCore, getError) {
        return __awaiter(this, void 0, void 0, function* () {
            publishRequestData.packagePaths = yield this.getPreparePackagePaths(publishDataCore);
            publishRequestData.sharedCloud = publishDataCore.sharedCloud;
            this.$logger.info("Starting publishing.");
            const response = yield this.$nsCloudServerBuildService.publish(publishRequestData);
            this.$logger.trace("Publish response", response);
            const publishResult = yield this.waitForCloudOperationToFinish(publishRequestData.cloudOperationId, response, { silent: true });
            this.$logger.trace("Publish result:", publishResult);
            if (publishResult.code || publishResult.errors) {
                const err = getError(publishResult, publishRequestData);
                err.stderr = publishResult.stderr;
                err.stdout = publishResult.stdout;
                err.cloudOperationId = publishRequestData.cloudOperationId;
                throw err;
            }
            this.$logger.info("Publishing finished successfully.");
        });
    }
    getPreparePackagePaths(publishData) {
        return __awaiter(this, void 0, void 0, function* () {
            const preparedPackagePaths = [];
            for (const packagePath of publishData.packagePaths) {
                preparedPackagePaths.push(this.$fs.exists(packagePath) ? yield this.$nsCloudUploadService.uploadToS3(packagePath, path_1.basename(packagePath)) : packagePath);
            }
            return preparedPackagePaths;
        });
    }
    validatePublishData(publishData) {
        if (!publishData.packagePaths || !publishData.packagePaths.length) {
            this.$nsCloudErrorsService.fail("Cannot upload without packages");
        }
        if (!publishData.projectDir) {
            this.$nsCloudErrorsService.fail("Cannot perform publish - projectDir is required.");
        }
    }
    getFormattedError(publishResult, regex) {
        const log = publishResult.stdout || publishResult.stderr || "";
        return _.uniq(log.match(regex)).join(os_1.EOL);
    }
}
CloudPublishService.ITMS_ERROR_REGEX = /\[Transporter Error Output\]:.*/g;
CloudPublishService.GENERAL_ERROR_REGEX = /\[!\].*/g;
exports.CloudPublishService = CloudPublishService;
$injector.register("nsCloudPublishService", CloudPublishService);
