"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const events_1 = require("events");
const uuid_1 = require("uuid");
const constants_1 = require("../constants");
class CloudService extends events_1.EventEmitter {
    constructor($nsCloudErrorsService, $fs, $httpClient, $logger, $constants, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService) {
        super();
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$fs = $fs;
        this.$httpClient = $httpClient;
        this.$logger = $logger;
        this.$constants = $constants;
        this.$nsCloudOperationFactory = $nsCloudOperationFactory;
        this.$nsCloudOutputFilter = $nsCloudOutputFilter;
        this.$nsCloudProcessService = $nsCloudProcessService;
        this.cloudOperations = Object.create(null);
        this.$nsCloudProcessService.attachToProcessExitSignals(this, this.cleanup.bind(this));
    }
    getServerOperationOutputDirectory(options) {
        return "";
    }
    sendCloudMessage(message) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const cloudOperation = this.cloudOperations[message.cloudOperationId];
            if (!cloudOperation) {
                this.$nsCloudErrorsService.fail(`Cloud operation with id: ${message.cloudOperationId} not found.`);
            }
            yield cloudOperation.cloudOperation.sendMessage(message);
        });
    }
    getServerResults(serverResult) {
        global.showErrorForStoppedCloudBuilds();
        return [];
    }
    executeCloudOperation(cloudOperationName, action) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const cloudOperationId = uuid_1.v4();
            try {
                this.$logger.info(`Starting ${cloudOperationName}. Cloud operation id: ${cloudOperationId}`);
                const result = yield action(cloudOperationId);
                yield this.cleanCloudOperation(cloudOperationId);
                return result;
            }
            catch (err) {
                yield this.cleanCloudOperation(cloudOperationId);
                err.cloudOperationId = cloudOperationId;
                throw err;
            }
        });
    }
    waitForCloudOperationToFinish(cloudOperationId, serverResponse, options) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const cloudOperationVersion = serverResponse.cloudOperationVersion || CloudService.CLOUD_OPERATION_VERSION_1;
            const cloudOperation = this.$nsCloudOperationFactory.create(cloudOperationVersion, cloudOperationId, serverResponse);
            if (options.parentCloudOperationId) {
                this.cloudOperations[options.parentCloudOperationId].children.push(cloudOperation);
            }
            this.cloudOperations[cloudOperationId] = { cloudOperation, children: [] };
            cloudOperation.on(constants_1.CloudCommunicationEvents.MESSAGE, (m) => {
                if (m.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_OUTPUT && !options.silent) {
                    const body = m.body;
                    let log = body.data;
                    if (cloudOperationVersion !== CloudService.CLOUD_OPERATION_VERSION_1) {
                        log = this.$nsCloudOutputFilter.filterBpcMetadata(log);
                    }
                    if (body.pipe === "stdout") {
                        this.printMessageOnTheSameLine(log);
                    }
                    else if (body.pipe === "stderr") {
                        this.$logger.error(log);
                    }
                }
                else if (m.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_SERVER_HELLO && !options.hideBuildMachineMetadata) {
                    const body = m.body;
                    if (body.hostName) {
                        this.$logger.info(`Build machine host name: ${body.hostName}`);
                    }
                }
                this.emit(constants_1.CloudCommunicationEvents.MESSAGE, m);
            });
            try {
                yield cloudOperation.init();
            }
            catch (err) {
                this.$logger.trace(err);
                yield cloudOperation.cleanup();
                throw new Error(this.failedToStartError);
            }
            try {
                const result = yield cloudOperation.waitForResult();
                yield cloudOperation.cleanup();
                return result;
            }
            catch (err) {
                this.$logger.trace(err);
                yield cloudOperation.cleanup();
                throw new Error(this.failedError);
            }
        });
    }
    downloadServerResults(serverResult, outputOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const destinationDir = this.getServerOperationOutputDirectory(outputOptions);
            this.$fs.ensureDirectoryExists(destinationDir);
            const serverResultObjs = this.getServerResults(serverResult);
            let targetFileNames = [];
            for (const serverResultObj of serverResultObjs) {
                this.$logger.info(`Result url: ${serverResultObj.fullPath}`);
                let filename = serverResultObj.filename;
                if (outputOptions.extension) {
                    filename = `${path.parse(filename).name}.${outputOptions.extension}`;
                }
                const targetFileName = path.join(destinationDir, filename);
                targetFileNames.push(targetFileName);
                const targetFile = this.$fs.createWriteStream(targetFileName);
                yield this.$httpClient.httpRequest({
                    url: serverResultObj.fullPath,
                    pipeTo: targetFile
                });
            }
            return targetFileNames;
        });
    }
    getCollectedLogs(cloudOperationId) {
        return this.cloudOperations[cloudOperationId].cloudOperation.getCollectedLogs();
    }
    cleanup() {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(_(this.cloudOperations).keys().map((id) => this.cleanCloudOperation(id)).value());
            this.cloudOperations = {};
        });
    }
    cleanCloudOperation(cloudOperationId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const cloudOperation = this.cloudOperations[cloudOperationId];
                if (!cloudOperation) {
                    return;
                }
                for (let child of cloudOperation.children) {
                    yield child.cleanup();
                    delete this.cloudOperations[child.id];
                }
                yield cloudOperation.cloudOperation.cleanup();
                delete this.cloudOperations[cloudOperationId];
            }
            catch (err) {
                this.$logger.error(`Cloud operation ${cloudOperationId} failed with error: ${err.message}`);
            }
        });
    }
    printMessageOnTheSameLine(msg) {
        const logger = this.$logger;
        if (logger.printInfoMessageOnSameLine) {
            logger.printInfoMessageOnSameLine(msg);
        }
        else {
            this.$logger.info(msg, { [this.$constants.LoggerConfigData.skipNewLine]: true });
        }
    }
}
CloudService.CLOUD_OPERATION_VERSION_1 = "v1";
exports.CloudService = CloudService;
