"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const constants_1 = require("../constants");
class EulaServiceBase {
    constructor($fs, $httpClient, $nsCloudLockService, $logger, $nsCloudHashService, $settingsService, $userSettingsService, $nsCloudTempService) {
        this.$fs = $fs;
        this.$httpClient = $httpClient;
        this.$nsCloudLockService = $nsCloudLockService;
        this.$logger = $logger;
        this.$nsCloudHashService = $nsCloudHashService;
        this.$settingsService = $settingsService;
        this.$userSettingsService = $userSettingsService;
        this.$nsCloudTempService = $nsCloudTempService;
        this.isEulaDownloadedInCurrentProcess = false;
    }
    getEulaData() {
        return this.getEulaDataWithCache();
    }
    getEulaDataWithCache() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.downloadLatestEula();
            return this.getEulaDataCore();
        });
    }
    acceptEula() {
        return __awaiter(this, void 0, void 0, function* () {
            const currentEulaHash = yield this.getCurrentEulaHash();
            this.$logger.trace(`Accepting EULA with hash ${currentEulaHash}.`);
            yield this.setAcceptedEulaHash(currentEulaHash);
            this.$logger.info("EULA has been accepted.");
        });
    }
    getEulaDataCore() {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldAcceptEula = yield this.getShouldAcceptLocalEula();
            this.$logger.trace(`Checked ${this.getEulaFileName()} state - shouldAcceptEula is ${shouldAcceptEula}.`);
            return {
                url: constants_1.EulaConstants.eulaUrl,
                shouldAcceptEula
            };
        });
    }
    getShouldAcceptLocalEula() {
        return __awaiter(this, void 0, void 0, function* () {
            const acceptedEulaHash = yield this.getAcceptedEulaHash();
            if (!acceptedEulaHash) {
                this.$logger.trace(`Checking ${this.getPathToEula()} state: no EULA has been accepted so far.`);
                return true;
            }
            const currentEulaHash = yield this.getLocalEulaHash();
            if (!currentEulaHash) {
                this.$logger.trace(`Checking ${this.getPathToEula()} state: no local copy of EULA found - as user had already accepted previous version of the EULA, consider it as the current one, so no need to accept new.`);
                return false;
            }
            this.$logger.trace(`Checking ${this.getPathToEula()} state: acceptedEulaHash is ${acceptedEulaHash}, currentEulaHash is ${currentEulaHash}.`);
            return acceptedEulaHash !== currentEulaHash;
        });
    }
    downloadLatestEula(opts = { forceDownload: false }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isEulaDownloadedInCurrentProcess) {
                this.$logger.trace("EULA is already downloaded in current process. Skip new download.");
                return;
            }
            try {
                const tempEulaPath = yield this.$nsCloudTempService.path({ prefix: "eula", suffix: ".pdf" });
                this.$logger.trace(`Downloading EULA to ${this.getPathToEula()}.`);
                const eulaFstat = this.getEulaFsStat();
                const result = yield this.$httpClient.httpRequest({
                    url: this.getEulaUrl(),
                    pipeTo: this.$fs.createWriteStream(tempEulaPath),
                    headers: eulaFstat && !opts.forceDownload ? { "If-Modified-Since": eulaFstat.mtime.toUTCString() } : {},
                    timeout: constants_1.EulaConstants.timeout
                });
                if (result.response.statusCode !== 304) {
                    const lockFilePath = this.getLockFilePath("download.lock");
                    yield this.$nsCloudLockService.executeActionWithLock(() => __awaiter(this, void 0, void 0, function* () {
                        this.$logger.trace(`Successfully downloaded EULA to ${tempEulaPath}.`);
                        this.$fs.copyFile(tempEulaPath, this.getPathToEula());
                        this.$logger.trace(`Successfully copied EULA to ${this.getPathToEula()}.`);
                    }), lockFilePath, this.getLockOptions());
                }
                this.isEulaDownloadedInCurrentProcess = true;
            }
            catch (err) {
                if (opts.forceDownload) {
                    this.$logger.trace("Unable to download latest EULA, but will rethrow the error as requested by caller. Error is:", err);
                    throw err;
                }
                this.$logger.trace("Unable to download latest EULA. Error is:", err);
            }
        });
    }
    getCurrentEulaHash() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.downloadLatestEula({ forceDownload: true });
            const localEulaHash = yield this.getLocalEulaHash();
            this.$logger.trace(`Downloaded latest ${this.getPathToEula()}, its hash is: ${localEulaHash}.`);
            return localEulaHash;
        });
    }
    getLocalEulaHash() {
        return __awaiter(this, void 0, void 0, function* () {
            const pathToEula = this.getPathToEula();
            const localEulaHash = this.$fs.exists(pathToEula) && (yield this.$nsCloudHashService.getLocalFileHash(pathToEula));
            this.$logger.trace(`Downloaded latest ${this.getPathToEula()}, its hash is: ${localEulaHash}.`);
            return localEulaHash;
        });
    }
    getLockOptions() {
        return {
            retryWait: 100,
            retries: 100,
            stale: 13 * 1000
        };
    }
    getLockFilePath(lockFileName) {
        const dirName = path.dirname(this.getPathToEula());
        const eulaName = path.parse(this.getPathToEula()).name;
        return path.join(dirName, `${eulaName}.${lockFileName}`);
    }
    getAcceptedEulaHash() {
        const propertyName = this.getAcceptedEulaHashPropertyName();
        return this.$userSettingsService.getSettingValue(propertyName);
    }
    setAcceptedEulaHash(hash) {
        const propertyName = this.getAcceptedEulaHashPropertyName();
        return this.$userSettingsService.saveSetting(propertyName, hash);
    }
    getEulaFsStat() {
        return this.$fs.exists(this.getPathToEula()) && this.$fs.getFsStats(this.getPathToEula());
    }
    getPathToEula() {
        return path.join(this.$settingsService.getProfileDir(), this.getEulaFileName());
    }
}
exports.EulaServiceBase = EulaServiceBase;
