"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const nativescript_doctor_1 = require("nativescript-doctor");
const helpers_1 = require("../helpers");
class GitService {
    constructor($childProcess, $fs, $hostInfo, $logger, $options, $nsCloudUserService, $nsCloudHashService) {
        this.$childProcess = $childProcess;
        this.$fs = $fs;
        this.$hostInfo = $hostInfo;
        this.$logger = $logger;
        this.$options = $options;
        this.$nsCloudUserService = $nsCloudUserService;
        this.$nsCloudHashService = $nsCloudHashService;
    }
    gitPushChanges(projectSettings, remoteUrl, codeCommitCredential, repositoryState) {
        return __awaiter(this, void 0, void 0, function* () {
            this.cleanLocalRepositories();
            if (repositoryState && repositoryState.isNewRepository) {
                this.deleteLocalRepository(projectSettings);
            }
            if (!this.isGitRepository(projectSettings)) {
                yield this.gitInit(projectSettings);
            }
            const isRemoteAdded = yield this.gitCheckIfRemoteIsAdded(projectSettings, GitService.REMOTE_NAME);
            if (isRemoteAdded) {
                const isGitRemoteCorrect = yield this.isGitRemoteSetToCorrectUrl(projectSettings, remoteUrl);
                if (!isGitRemoteCorrect) {
                    yield this.gitSetRemoteUrl(projectSettings, GitService.REMOTE_NAME, remoteUrl);
                }
            }
            else {
                yield this.gitRemoteAdd(projectSettings, remoteUrl);
            }
            yield this.ensureGitIgnoreExists(projectSettings.projectDir);
            const isAutocrlfFalse = yield this.gitCheckIfAutocrlfIsFalse(projectSettings);
            if (!isAutocrlfFalse) {
                this.$logger.trace("Setting core.autocrlf to false");
                yield this.executeCommand(projectSettings, ["config", "--local", "core.autocrlf", "false"]);
            }
            yield this.configureEnvironment(projectSettings, remoteUrl);
            const statusResult = yield this.gitStatus(projectSettings);
            this.$logger.trace(`Result of git status: ${statusResult}.`);
            if (this.hasNothingToCommit(statusResult.stdout)) {
                this.$logger.trace("Nothing to commit. Just push force the branch.");
                yield this.gitPush(projectSettings, codeCommitCredential);
                return;
            }
            yield this.gitAdd(projectSettings);
            yield this.gitCommit(projectSettings);
            yield this.gitPush(projectSettings, codeCommitCredential);
        });
    }
    cleanLocalRepositories() {
        if (!this.$fs.exists(this.getGitDirBasePath())) {
            return;
        }
        const entries = this.$fs.readDirectory(this.getGitDirBasePath())
            .map(entry => path.join(this.getGitDirBasePath(), entry));
        _.each(entries, entry => {
            const stats = this.$fs.getFsStats(entry);
            if (new Date().getTime() - stats.ctime.getTime() > GitService.MAX_TIME_FOR_UNUSED_REPOSITORY) {
                this.$fs.deleteDirectory(entry);
            }
        });
    }
    isGitRemoteSetToCorrectUrl(projectSettings, remoteUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.executeCommand(projectSettings, ["remote", "-v"]);
            return result.stdout.indexOf(remoteUrl.httpRemoteUrl) !== -1;
        });
    }
    configureEnvironment(projectSettings, remoteUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const gitVersion = yield nativescript_doctor_1.sysInfo.getGitVersion();
            if (gitVersion) {
                const match = gitVersion.match(/^(\d+?)\.(\d+?)\.\d+.*$/);
                const gitMajorVersion = ~~match[1];
                const gitMinorVersion = ~~match[2];
                if (this.$hostInfo.isWindows) {
                    if (gitMajorVersion === GitService.MINIMAL_GIT_MAJOR_VERSION &&
                        gitMinorVersion >= GitService.MINIMAL_GIT_MINOR_VERSION) {
                        yield this.executeCommand(projectSettings, ["config", "--local", `credential.${remoteUrl.httpRemoteUrl}.helper`, ""]);
                    }
                    else {
                        throw new Error(`Unsupported Git version: ${gitVersion}. The minimal supported version is 2.9.0. Please update.`);
                    }
                }
            }
            yield this.executeCommand(projectSettings, ["config", "--local", "credential.helper", this.getCredentialHelperPath()]);
            yield this.executeCommand(projectSettings, ["config", "--local", "credential.UseHttpPath", "true"]);
        });
    }
    gitInit(projectSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executeCommand(projectSettings, ["init"]);
        });
    }
    gitCommit(projectSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executeCommand(projectSettings, ["commit", `-m "cloud-commit-${new Date().toString()}"`]);
        });
    }
    gitAdd(projectSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executeCommand(projectSettings, ["add", projectSettings.projectDir]);
        });
    }
    gitStatus(projectSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executeCommand(projectSettings, ["status"]);
        });
    }
    gitPush(projectSettings, codeCommitCredential) {
        return __awaiter(this, void 0, void 0, function* () {
            const env = _.assign({}, process.env, {
                AWS_ACCESS_KEY_ID: codeCommitCredential.accessKeyId,
                AWS_SECRET_ACCESS_KEY: codeCommitCredential.secretAccessKey,
                AWS_SESSION_TOKEN: codeCommitCredential.sessionToken
            });
            return this.executeCommand(projectSettings, ["push", "--force", GitService.REMOTE_NAME, GitService.BRANCH_NAME], { env, cwd: projectSettings.projectDir });
        });
    }
    gitRemoteAdd(projectSettings, remoteUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executeCommand(projectSettings, ["remote", "add", GitService.REMOTE_NAME, remoteUrl.httpRemoteUrl]);
        });
    }
    gitSetRemoteUrl(projectSettings, remoteName, remoteUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.executeCommand(projectSettings, ["remote", "set-url", remoteName, remoteUrl.httpRemoteUrl]);
        });
    }
    gitCheckIfRemoteIsAdded(projectSettings, remoteName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.executeCommand(projectSettings, ["remote", "get-url", remoteName]);
                return true;
            }
            catch (err) {
                this.$logger.trace("Error while checking if remote is added: ", err);
                return false;
            }
        });
    }
    gitCheckIfAutocrlfIsFalse(projectSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.executeCommand(projectSettings, ["config", "core.autocrlf"]);
                return result && result.stdout && result.stdout.toString().trim().toLowerCase() === "false";
            }
            catch (err) {
                this.$logger.trace("Error while checking if core.autocrlf is false: ", err);
                return false;
            }
        });
    }
    executeCommand(projectSettings, args, options, spawnFromEventOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            options = options || { cwd: projectSettings.projectDir };
            const gitDir = this.getGitDirPath(projectSettings);
            this.$fs.ensureDirectoryExists(gitDir);
            const command = yield this.getGitFilePath();
            args = [`--git-dir=${gitDir}`, `--work-tree=${projectSettings.projectDir}`].concat(args);
            return this.$childProcess.spawnFromEvent(command, args, "close", options, spawnFromEventOptions);
        });
    }
    deleteLocalRepository(projectSettings) {
        if (this.isGitRepository(projectSettings)) {
            this.$fs.deleteDirectory(this.getGitDirPath(projectSettings));
        }
    }
    isGitRepository(projectSettings) {
        return this.$fs.exists(this.getGitDirPath(projectSettings));
    }
    getGitDirPath(projectSettings) {
        return path.join(this.getGitDirBasePath(), this.getGitDirName(projectSettings));
    }
    getGitDirBasePath() {
        return path.join(this.$options.profileDir, GitService.GIT_DIR_NAME);
    }
    getGitDirName(projectSettings) {
        const dirName = `${this.$nsCloudUserService.getUser().email}_${projectSettings.projectDir}_${projectSettings.projectId}_${GitService.GIT_REPO_VERSION}`;
        return this.$nsCloudHashService.getHash(dirName, { algorithm: "sha1" });
    }
    getGitFilePath() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.gitFilePath) {
                this.gitFilePath = yield nativescript_doctor_1.sysInfo.getGitPath();
                this.$logger.trace(`Path to git is: ${this.gitFilePath}.`);
                if (!this.gitFilePath) {
                    throw new Error("Git Installation Not Found. Install Git to improve the speed of cloud builds.");
                }
            }
            return this.gitFilePath;
        });
    }
    ensureGitIgnoreExists(projectDir) {
        const gitIgnorePath = path.join(projectDir, GitService.GIT_IGNORE_FILE_NAME);
        this.$logger.trace(`Ensure ${gitIgnorePath} exists.`);
        if (!this.$fs.exists(gitIgnorePath)) {
            this.$logger.trace(`${gitIgnorePath} does not exist. Creating a default one.`);
            const gitIgnoreResourceFile = path.join(this.getGitResourceFolder(), GitService.TEMPLATE_GIT_IGNORE_FILE_NAME);
            this.$fs.copyFile(gitIgnoreResourceFile, gitIgnorePath);
        }
    }
    getCredentialHelperPath() {
        const credentialHelperPath = path.join(this.getGitResourceFolder(), "aws-credential-helper", "credential-helper");
        return helpers_1.fromWindowsRelativePathToUnix(credentialHelperPath);
    }
    getGitResourceFolder() {
        return path.join(__dirname, "..", "..", "resources", "git");
    }
    hasNothingToCommit(stdout) {
        return /nothing to commit/.test(stdout);
    }
}
GitService.REMOTE_NAME = "cloud";
GitService.BRANCH_NAME = "master";
GitService.GIT_DIR_NAME = ".cloud-git";
GitService.GIT_IGNORE_FILE_NAME = ".gitignore";
GitService.TEMPLATE_GIT_IGNORE_FILE_NAME = "template-.gitignore";
GitService.MAX_TIME_FOR_UNUSED_REPOSITORY = 1000 * 3600 * 24 * 30;
GitService.MINIMAL_GIT_MAJOR_VERSION = 2;
GitService.MINIMAL_GIT_MINOR_VERSION = 9;
GitService.GIT_REPO_VERSION = "v1";
exports.GitService = GitService;
$injector.register("nsCloudGitService", GitService);
