"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
const url_1 = require("url");
class ServerServicesProxy {
    constructor($nsCloudErrorsService, $httpClient, $logger, $nsCloudConfigManager, $nsCloudUserService) {
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$httpClient = $httpClient;
        this.$logger = $logger;
        this.$nsCloudConfigManager = $nsCloudConfigManager;
        this.$nsCloudUserService = $nsCloudUserService;
        this.serverConfig = this.$nsCloudConfigManager.getCurrentConfigData();
    }
    call(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const host = this.getServiceDomainName(options.serviceName);
            const finalUrlPath = this.getUrlPath(options.serviceName, options.urlPath);
            const headers = options.headers || Object.create(null);
            const authScheme = this.getAuthScheme(options.serviceName);
            if (!_.has(headers, constants_1.HTTP_HEADERS.AUTHORIZATION) && this.$nsCloudUserService.hasUser()) {
                headers[constants_1.HTTP_HEADERS.AUTHORIZATION] = `${authScheme} ${this.$nsCloudUserService.getUserData().accessToken}`;
            }
            const namespace = this.$nsCloudConfigManager.getServiceValueOrDefault(options.serviceName, "namespace");
            if (namespace) {
                headers[constants_1.HTTP_HEADERS.X_NS_NAMESPACE] = namespace;
                if (this.$nsCloudUserService.hasUser()) {
                    const userData = this.$nsCloudUserService.getUserData();
                    headers[constants_1.HTTP_HEADERS.X_NS_INSTANCE_ID] = userData.instanceId || "";
                }
            }
            if (options.accept) {
                headers[constants_1.HTTP_HEADERS.ACCEPT] = options.accept;
            }
            const reqProto = this.getServiceProto(options.serviceName);
            const reqUrl = new url_1.URL(finalUrlPath, `${reqProto}://${host}`);
            let requestOpts = {
                url: reqUrl.toString(),
                method: options.method,
                headers: headers,
                pipeTo: options.resultStream,
                timeout: options.timeout
            };
            if (options.bodyValues) {
                if (options.bodyValues.length > 1) {
                    throw new Error("TODO: CustomFormData not implemented");
                }
                let theBody = options.bodyValues[0];
                requestOpts.body = theBody.value;
                requestOpts.headers[constants_1.HTTP_HEADERS.CONTENT_TYPE] = theBody.contentType;
            }
            let response;
            try {
                response = yield this.$httpClient.httpRequest(requestOpts);
            }
            catch (err) {
                if (err.response && err.response.statusCode === 402) {
                    this.$nsCloudErrorsService.fail(JSON.parse(err.body).message);
                }
                throw err;
            }
            this.$logger.debug("%s (%s %s) returned %d", finalUrlPath, options.method, options.urlPath, response.response.statusCode);
            try {
                const resultValue = options.accept === constants_1.CONTENT_TYPES.APPLICATION_JSON ? JSON.parse(response.body) : response.body;
                return resultValue;
            }
            catch (err) {
                this.$logger.trace("Error while trying to parse body: ", err);
                throw new Error(`Server returned unexpected response: ${response.body}`);
            }
        });
    }
    getServiceProto(serviceName) {
        return this.$nsCloudConfigManager.getServiceValueOrDefault(serviceName, "serverProto");
    }
    getUrlPath(serviceName, urlPath) {
        const apiVersion = this.$nsCloudConfigManager.getServiceValueOrDefault(serviceName, "apiVersion");
        let result;
        if (!apiVersion) {
            result = urlPath;
        }
        else {
            result = `${apiVersion}/${urlPath}`;
        }
        return result.startsWith("/") ? result : `/${result}`;
    }
    getAuthScheme(serviceName) {
        return constants_1.BEARER_AUTH_SCHEME;
    }
    getServiceDomainName(serviceName) {
        return this.$nsCloudConfigManager.getServiceDomainName(serviceName);
    }
}
exports.ServerServicesProxy = ServerServicesProxy;
$injector.register("nsCloudServerServicesProxy", ServerServicesProxy);
