"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
class VersionService {
    constructor($httpClient, $logger, $nsCloudPolyfillService, $nsCloudConfigurationService, $projectDataService) {
        this.$httpClient = $httpClient;
        this.$logger = $logger;
        this.$nsCloudPolyfillService = $nsCloudPolyfillService;
        this.$nsCloudConfigurationService = $nsCloudConfigurationService;
        this.$projectDataService = $projectDataService;
    }
    get $constants() {
        return this.$nsCloudPolyfillService.getPolyfillObject("constants", {
            NATIVESCRIPT_PROPS_INTERNAL_DELIMITER: "."
        });
    }
    getCliVersion(runtimeVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const cloudConfigData = this.$nsCloudConfigurationService.getCloudConfigurationData();
                const cliCloudVersionOverride = cloudConfigData && cloudConfigData.tnsCliCloudVersion || process.env.TNS_CLI_CLOUD_VERSION;
                const latestMatchingVersion = cliCloudVersionOverride || (yield this.getLatestMatchingVersion("nativescript", runtimeVersion));
                if (!latestMatchingVersion) {
                    throw new Error("Cannot find CLI versions.");
                }
                return latestMatchingVersion;
            }
            catch (err) {
                throw new Error(`Unable to determine CLI version for cloud build based on project's runtime version: ${runtimeVersion}. Error is: ${err.message}`);
            }
        });
    }
    getProjectRuntimeVersion(projectDir, platform) {
        return __awaiter(this, void 0, void 0, function* () {
            const runtimePackageName = `tns-${platform.toLowerCase()}`;
            const runtimeVersion = this.$projectDataService.getNSValue(projectDir, `${runtimePackageName}${this.$constants.NATIVESCRIPT_PROPS_INTERNAL_DELIMITER}version`);
            if (!runtimeVersion) {
                throw new Error(`Unable to find runtime version for package ${runtimePackageName}.`);
            }
            return runtimeVersion;
        });
    }
    getLatestMatchingVersion(packageName, dependentPackageName) {
        return __awaiter(this, void 0, void 0, function* () {
            const patchRange = this.getVersionRangeWithTilde({
                versionString: dependentPackageName,
                shouldRespectPatchVersion: true
            });
            const generalRange = this.getVersionRangeWithTilde({
                versionString: dependentPackageName
            });
            const versions = yield this.getVersionsFromNpm(packageName);
            if (versions.length) {
                return semver.maxSatisfying(versions, patchRange) || semver.maxSatisfying(versions, generalRange);
            }
            return null;
        });
    }
    getVersionsFromNpm(packageName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const packageData = yield this.getPackageDataFromNpm(packageName);
                return _.keys(packageData.versions);
            }
            catch (err) {
                this.$logger.trace(`Unable to get versions of ${packageName} from npm. Error is: ${err.message}.`);
                return [];
            }
        });
    }
    getPackageDataFromNpm(packageName) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.$httpClient.httpRequest(`http://registry.npmjs.org/${packageName}`);
            return JSON.parse(response.body);
        });
    }
    getVersionRangeWithTilde(versionRangeOptions) {
        const prerelease = semver.prerelease(versionRangeOptions.versionString);
        const prereleaseString = prerelease ? "-0" : "";
        const major = semver.major(versionRangeOptions.versionString);
        const minor = semver.minor(versionRangeOptions.versionString);
        const patch = versionRangeOptions.shouldRespectPatchVersion ? semver.patch(versionRangeOptions.versionString) : "0";
        return `~${major}.${minor}.${patch}${prereleaseString}`;
    }
}
exports.VersionService = VersionService;
$injector.register("nsCloudVersionService", VersionService);
